@@ -5,6 +5,13 @@ import glob
 import astropy_stark.cream_lcplot as cream_plot
 import astropy_stark.cream_plotlibrary as cpl
 import matplotlib.pylab as plt
+import multiprocessing as mp
+import time
+import itertools
+import corner
+from scipy.stats import pearsonr
+import seaborn as sns; sns.set(style="ticks", color_codes=True)
+
 
 class pycecream:
     '''
@@ -42,8 +49,9 @@ class pycecream:
         print('pycecream path... ' + self.module_path)
 
         #convention parameters
-        self.output_directory = 'pycecream'
+        self.project_folder = 'pycecream'
         self.append_date_to_output_directory = False
+        self.save_ALL_parameters = True
 
         #global non-fitted parameters
         self.redshift = 0.0
@@ -102,16 +110,21 @@ class pycecream:
         :return:
         '''
         #make a directory into which to store the cream results
-        dir_pycecream = self.output_directory
+        dir_pycecream = self.project_folder
         if self.append_date_to_output_directory is True:
-            dir_pycecream = self.output_directory+'_'+str(pd.datetime.today().strftime("%d_%m_%Y"))+'_'
+            dir_pycecream = self.project_folder+'_'+str(pd.datetime.today().strftime("%d_%m_%Y"))+'_'
             child_dirs = next(os.walk('.'))[1]
             number_of_pyceream_dirs = len( [c for c in child_dirs if dir_pycecream in c] )
             dir_pycecream = dir_pycecream+str(number_of_pyceream_dirs)
 
         self.dir_pycecream = dir_pycecream
         self.dir_sim = 'simulation_files'
-        os.mkdir(self.dir_pycecream)
+        try:
+            os.mkdir(self.dir_pycecream)
+        except:
+            raise Exception('directory...'+self.dir_pycecream+' already exists. '
+                                                              'Please choose another using the '
+                                                              'self.dir_pycecream variable')
         os.mkdir(self.dir_pycecream+'/'+self.dir_sim)
 
         #copy fortran files to pycecream directory
@@ -138,8 +151,7 @@ class pycecream:
                tophat_width = None,
                tophat_width_step = None,
                tophat_width_prior=[0.0, -0.1],
-
-    ):
+               background_polynomials = None):
         '''
         This is the go to command to add a new light curve into the
         simulation.
@@ -155,8 +167,13 @@ class pycecream:
         :param vertical_scaling_start: as above but for vertical scaling parameter.
         :param background_offset_prior: [mean,sd] of gaussian prior. Leave as None to ignore priors
         :param vertical_scaling_prior: as above but for the vertical scaling parameter
+        :param background_polynomials: add variable background list of starting coefficients for each level of polynomial
+        (advise [0.1,0.1] to add a quadratic polynomial )
         :return:
         '''
+        #assertion errors
+        assert(type(share_previous_lag) == bool)
+
         #set up the directory structure if first call
         if self.count_lightcurves == 0:
             self.setup_directory_structure()
@@ -229,7 +246,8 @@ class pycecream:
                                   background_offset_start,
                                   vertical_scaling_start,
                                   background_offset_prior,
-                                  vertical_scaling_prior
+                                  vertical_scaling_prior,
+                                  background_polynomials
                                   ],
                      index=['name', 'type', 'wavelength', 'noise model',
                             'extra variance prior','multiplicative errorbar prior',
@@ -243,7 +261,8 @@ class pycecream:
                             'tophat width prior cent',
                             'tophat width prior width',
                             'background offset start','vertical scaling start',
-                            'background offset prior','vertical scaling prior']).T
+                            'background offset prior','vertical scaling prior',
+                            'background_polynomials']).T
 
         self.lightcurve_input_params = pd.DataFrame(pd.concat([self.lightcurve_input_params,df]))
         self.lightcurve_input_params['wavelength']= \
@@ -273,12 +292,15 @@ class pycecream:
         # lines for each of the input parameters in creaminpar.par (dont change)
         idcos = 37
         idmdot = 31
+        idmass = 32
+        idefficiency = 25
         idslope = 70
         idsig = 22
         idnits = 10
         idplot = 4
         idlaglim = 13
         idhifreq = 9
+        idallparsave = 2
 
         #modify and write the creaminpar.par file as needed
         with open(self.dir_pycecream+'/creaminpar.par') as f:
@@ -288,8 +310,10 @@ class pycecream:
             content[0] = './'+np.str(self.dir_sim)
             f.close()
         content[idcos] = np.str(self.p_inclination)
+        content[idefficiency] = np.str(self.bh_efficieny)
         content[idcos+1] = np.str(self.p_inclination_step)
         content[idmdot] = np.str(self.p_accretion_rate)
+        content[idmass] = np.str(self.bh_mass)
         content[idmdot+3] = np.str(self.p_accretion_rate_step)
         content[idslope] = np.str(self.p_viscous_slope)
         content[idslope+2] = np.str(self.p_viscous_slope_step)
@@ -316,6 +340,10 @@ class pycecream:
         content[idsig - 1] = a
         content[idsig - 2] = turn_on_multiplicative_noise
 
+        # decide whether to record all Fourier parameters (possible bad for storage space)
+        if self.save_ALL_parameters == True:
+            content[idallparsave] = 'F T'
+
         #write updated creaminpar.par file
         f = open(self.dir_pycecream+'/creaminpar.par', 'w')
         for fn in content:
@@ -323,6 +351,7 @@ class pycecream:
         f.close()
 
 
+
     def set_creamnames(self):
         '''
         set the creamnames.dat file summarising the file names and wavelengths
@@ -411,6 +440,37 @@ class pycecream:
         f.close()
 
 
+    def set_background_polynomials(self):
+        '''
+        allows a smooth background function to be fitted to each light curve alongside lamppost model
+        If background_polynomial parameters not configured then this function does nothing
+        :return:
+        '''
+        all_polys = self.lightcurve_input_params['background_polynomials']
+        NPpoly = 0
+        Nwavs = len(self.lightcurve_input_params)
+        # need a 2D array of polynomial coefficients but may user specifies these as a list
+        # of variable lengths take maximum length and set other coeffs to 0
+        for p in all_polys:
+            if p is not None:
+                NPpoly = max(NPpoly,len(p))
+        poly_coefs = np.zeros((Nwavs,NPpoly))
+        for i in range(Nwavs):
+            if all_polys.iloc[i] is not None:
+                Npoly_now = len(all_polys.iloc[i])
+                poly_coefs[i,:Npoly_now] = all_polys.iloc[i]
+
+        #populate the cream file creaminpar_bg.par
+        if NPpoly > 0:
+            f = open(self.dir_pycecream + '/' + self.dir_sim + '/creaminpar_bg.par', 'w')
+            f.write(str(NPpoly)+'\n')
+            f.write(' '.join(['0.0']*NPpoly)+'\n')
+            for i in range(Nwavs):
+                x = list(poly_coefs[i,:])
+                f.write(' '.join([str(xx) for xx in x])+'\n')
+                f.write(' '.join(['0.0']*NPpoly)+'\n') #no priors allowed
+
+
 
 
     def set_var(self):
@@ -432,7 +492,7 @@ class pycecream:
 
 
 
-    def run(self):
+    def run(self,ncores = 1):
         '''
         run the cream code. Make sure input above is correct first
         :return:
@@ -443,6 +503,7 @@ class pycecream:
         self.set_creamnames()
         self.set_tophat()
         self.set_var()
+        self.set_background_polynomials()
         self.set_start_offsert_vertical()
         self.set_priors()
 
@@ -451,10 +512,23 @@ class pycecream:
         self.lightcurve_input_params.to_csv('./simulation_files/input_lightcurve_settings.csv')
 
         #compile and run
+        # with mp.Pool(ncores) as p:
+        #    p.map(_run_cmd,[('./creamrun.exe',self.dir_pwd)]*ncores)
+
         os.system(self.fortran_compile_command)
-        os.system('./creamrun.exe')
+        jobs = []
+        for i in range(ncores):
+            p = mp.Process(target = _run_cmd, args=('./creamrun.exe',))
+            jobs.append(p)
+            p.start()
+            time.sleep(0.4)
+        #wait for jobs to finish before continuing
+        for j in jobs:
+            j.join()
         os.chdir(self.dir_pwd)
 
+    #def run_parallel(self,ncores=1):
+
 
     def get_simulation_dir(self,location=None):
         '''
@@ -478,43 +552,103 @@ class pycecream:
         '''
         #locate the simulation results
         simulation_dir = self.get_simulation_dir(location=location)
-        lcnames = self.lightcurve_input_params['name']
-        results_dir = glob.glob(simulation_dir + '/simulation_files/output_2*')[0]
-        '''
-        Begin loading the simulation results from the various stored locations
-        '''
+        lcnames = list(self.lightcurve_input_params['name'])
+        n_lightcurves = len(lcnames)
+        results_dir_list = glob.glob(simulation_dir + '/simulation_files/output_2*')
 
-        #load the disk parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars.dat')
-        p_output_names = ['Mdot','cos i','Tr_alpha']
-        p_output = dat_output[:,[2,3,4]]
+        self.output_parameters = pd.DataFrame()
+        idx_chain = 0
+        for results_dir in results_dir_list:
+            '''
+            Begin loading the simulation results from the various stored locations
+            '''
 
-        #load the stretch, offset and multiplicative error bar rescaling parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars2.dat')
-        p_output_names = p_output_names + ['stretch '+l for l in lcnames] + \
-                         ['offset '+l for l in lcnames] + \
-                         ['noise m '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output[:,:3*self.count_lightcurves]))
+            #load the disk parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars.dat')
+            p_output_names = ['disk Mdot','disk cos i','disk Tr_alpha']
+            p_output = dat_output[:,[2,3,4]]
 
-        #load the extra variance noise parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars_varexpand.dat')
-        p_output_names = p_output_names + \
-                         ['noise var '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output))
+            #load the stretch, offset and multiplicative error bar rescaling parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars2.dat')
+            p_output_names = p_output_names + ['stretch '+l for l in lcnames] + \
+                             ['offset '+l for l in lcnames] + \
+                             ['noise m '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output[:,:3*self.count_lightcurves]))
 
-        #load the top hat centroid parameters
-        dat_output = np.loadtxt(results_dir + '/outputpars_th.dat')
-        p_output_names = p_output_names + \
-                         ['top hat centroid '+l for l in lcnames] + \
-                         ['top hat width '+l for l in lcnames]
-        p_output = np.hstack((p_output,dat_output[:,:2*self.count_lightcurves]))
+            #load the extra variance noise parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars_varexpand.dat')
+            p_output_names = p_output_names + \
+                             ['noise var '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output))
 
-        #save all results for each MCMC iteration to a pandas data frame
-        self.output_parameters = pd.DataFrame(data = p_output,columns = p_output_names)
+            #load the top hat centroid parameters
+            dat_output = np.loadtxt(results_dir + '/outputpars_th.dat')
+            p_output_names = p_output_names + \
+                             ['top hat centroid '+l for l in lcnames] + \
+                             ['top hat width '+l for l in lcnames]
+            p_output = np.hstack((p_output,dat_output[:,:2*self.count_lightcurves]))
 
+
+            #load the polynomial background parameters if present
+            file_polynomial_background = results_dir + '/outputpars_bgvary.dat'
+            polynomial_background_exists = os.path.isfile(file_polynomial_background)
+            if polynomial_background_exists is True:
+                dat_pbg = np.loadtxt(file_polynomial_background,ndmin=2)
+                nrows,ncols = np.shape(dat_pbg)
+                n_pbg = int(ncols/2)
+                p_pbg = dat_pbg[:,:n_pbg]
+                norder = int(n_pbg/n_lightcurves)
+                p_pbg_names = []
+                for i in range(n_lightcurves):
+                    p_pbg_names += [lcnames[i]+' ng polynomial order '+str(i2+1) for i2 in range(norder)]
+                p_output_names = p_output_names + p_pbg_names
+                p_output = np.hstack((p_output,p_pbg))
+
+            #save all results for each MCMC iteration to a pandas data frame
+            p_output = pd.DataFrame(data = p_output,columns = p_output_names)
+            p_output['chain number'] = idx_chain
+            self.output_parameters = self.output_parameters.append(p_output)
+            idx_chain += 1
         return(self.output_parameters)
 
 
+    def get_MCMC_fourier_chains(self,location = None):
+        '''
+        retrieve the fourier parameters
+        :param location:
+        :return:
+        '''
+        #locate the simulation results
+        simulation_dir = self.get_simulation_dir(location=location)
+
+        results_dir_list = glob.glob(simulation_dir + '/simulation_files/output_2*')
+
+        self.output_parameters = pd.DataFrame()
+        idx_chain = 0
+        for results_dir in results_dir_list:
+            if idx_chain == 1:
+                #try loading fourier chains (only do once)
+                fourier_info = np.loadtxt(results_dir + '/cream_gvalues.dat',skiprows = 1)
+                self.fourier_info = pd.DataFrame(fourier_info,columns = ['angular frequency','prior std',
+                                                                         'gvalue sine','gvalue cos'])
+                ang_freq = list(self.fourier_info['angular frequency'])
+                Nfreq = len(self.fourier_info)
+
+            dat_fourier = pd.DataFrame(np.loadtxt(results_dir + '/CREAM_allpars_BIG.dat')[:, :2*Nfreq])
+            fourier_cols_sine = ['sine '+str(af) for af in ang_freq]
+            fourier_cols_cos = ['cos ' + str(af) for af in ang_freq]
+            fourier_cols = []
+            for i in range(len(fourier_cols_sine)):
+                fourier_cols.append(fourier_cols_sine[i])
+                fourier_cols.append(fourier_cols_cos[i])
+            dat_fourier.columns = fourier_cols
+            dat_fourier['chain number'] = idx_chain
+            idx_chain += 1
+
+        return {'fourier_chains':dat_fourier,
+                'fourier_stats':fourier_info}
+
+
 
 
     def get_light_curve_fits(self,location=None):
@@ -524,7 +658,8 @@ class pycecream:
         '''
         #locate the simulation results
         simulation_dir = self.get_simulation_dir(location=location)
-        results_dir = glob.glob(simulation_dir + '/simulation_files/output_2*')[0]
+        results_dir_list = glob.glob(simulation_dir + '/simulation_files/output_2*')
+        ncores = len(results_dir_list)
         '''
         Begin loading the simulation results from the various stored locations
         '''
@@ -532,17 +667,28 @@ class pycecream:
         '''
         the merged models for each wavelength
         '''
-        count = 0
-        output_merged_model = {}
-        for tf in self.lightcurve_input_params['temporary file name'].values:
-            dat = np.loadtxt(results_dir+'/plots/merged_mod_'+tf+'.dat')
-            name = self.lightcurve_input_params['name'].values[count]
-            if count == 0:
-                output_merged_model['time']=dat[:,0]
-            output_merged_model[name+' model'] = dat[:,1]
-            output_merged_model[name+' uncerts'] = dat[:, 2]
-            count = count + 1
-        self.output_merged_model = pd.DataFrame(output_merged_model)
+        output_merged_model_chains = []
+        output_merged_uncertainties_chains = []
+        for results_dir in results_dir_list:
+            count = 0
+            output_merged_model = []
+            output_merged_uncertainties = []
+            for tf in self.lightcurve_input_params['temporary file name'].values:
+                dat = np.loadtxt(results_dir+'/plots/merged_mod_'+tf+'.dat')
+                output_merged_model += [dat[:,1]]
+                output_merged_uncertainties += [dat[:, 2]]
+                count = count + 1
+            output_merged_model_chains += [np.array(output_merged_model).T]
+            output_merged_uncertainties_chains += [np.array(output_merged_uncertainties).T]
+
+        names = list(self.lightcurve_input_params['name'])
+        y = np.mean(np.array(output_merged_model_chains),axis=0)
+        sig = np.sqrt(np.sum(np.array(output_merged_uncertainties_chains)**2,axis=0))/ncores
+        self.output_merged_model = {'time':dat[:,0],
+                                    'light curve':pd.DataFrame(y,columns = names),
+                                    'light curve uncertainties':pd.DataFrame(sig,columns = names)}
+
+
         '''
         the merged data points for each file (this just takes the
         original light curves and applies the errorbar rescaling,
@@ -552,11 +698,12 @@ class pycecream:
         count = 0
         output_merged_data = {}
         for tf in self.lightcurve_input_params['temporary file name'].values:
-            dat = np.loadtxt(results_dir+'/plots/merged_dat_'+tf+'.dat')
+            dat = np.loadtxt(results_dir_list[0]+'/plots/merged_dat_'+tf+'.dat')
             name = self.lightcurve_input_params['name'].values[count]
-            output_merged_data[name+' time']=dat[:,0]
-            output_merged_data[name+' data'] = dat[:,1]
-            output_merged_data[name+' uncerts'] = dat[:, 3]
+            output_merged_data[name] = dat[:,[0,1,3]]
+            #output_merged_data[name+' time']=dat[:,0]
+            #output_merged_data[name+' data'] = dat[:,1]
+            #output_merged_data[name+' uncerts'] = dat[:, 3]
             count = count + 1
         self.output_merged_data = output_merged_data
 
@@ -564,10 +711,19 @@ class pycecream:
         load the unmerged model (easier). This is just the model (with error bar rescalings)
         evaluated at each wavelength without rescaling relative to reference light curve
         '''
-        dat = np.loadtxt(results_dir+'/plots/modellc.dat')
-        dat_sig = np.loadtxt(results_dir + '/plots/modellc_sig.dat')
+        dat = []
+        dat_sig = []
+        count = 0
+        for results_dir in results_dir_list:
+            dat += [np.loadtxt(results_dir+'/plots/modellc.dat')]
+            dat_sig += [np.loadtxt(results_dir + '/plots/modellc_sig.dat')]
+            if count == 0:
+                output_model = {'time': dat[0][:, 0]}
+            count += 1
+        dat = np.mean(np.array(dat),axis=0)
+        dat_sig = np.sqrt(np.sum(np.array(dat_sig)**2,axis=0))/ncores
+
         count = 0
-        output_model = {'time':dat[:,0]}
         for name in self.lightcurve_input_params['name'].values:
             output_model[name+' model'] = dat[:,count+1]
             output_model[name + ' uncerts'] = dat_sig[:, count + 1]
@@ -578,10 +734,15 @@ class pycecream:
         '''
         append the driving light curve to the output_model dataframe
         '''
-        driver = np.loadtxt(results_dir+'/plots/modeldrive.dat')
+        drivemoditp = []
+        drivemodsigitp = []
         timemod = self.output_model['time'].values
-        drivemoditp = np.interp(timemod,driver[:,0],driver[:,1])
-        drivemodsigitp = np.interp(timemod,driver[:,0],driver[:,2])
+        for results_dir in results_dir_list:
+            driver = np.loadtxt(results_dir+'/plots/modeldrive.dat')
+            drivemoditp += [np.interp(timemod,driver[:,0],driver[:,1])]
+            drivemodsigitp += [np.interp(timemod,driver[:,0],driver[:,2])]
+        drivemoditp = np.mean(np.array(drivemoditp),axis=0)
+        drivemodsigitp = np.sqrt(np.sum(np.array(drivemodsigitp),axis=0))/ncores
         self.output_model.insert(loc=1,column='driver',value = drivemoditp)
         self.output_model.insert(loc=2, column='driver uncerts', value=drivemodsigitp)
 
@@ -706,7 +867,6 @@ class pycecream:
         ax1 = fig.add_subplot(111)
         for c in cols:
             c5 = c[:-5]
-            print(model.columns)
             flux_m = model[c].values
             flux_m_sort = flux_m[idsort]
             flux_d = data[c5+'data']
@@ -748,6 +908,295 @@ class pycecream:
 
 
 
+class dream:
+
+    def __init__(self,Niterations = 1000):
+        '''
+        Simplified version of pycecream to create a single light combined
+        light curve from multiple observatories at a single wavelength
+        multiple telescopes at a single wavelength
+        '''
+        self.pc = pycecream()
+        self.N_iterations = Niterations
+        self.numlc = 0
+        self._op = None
+        self.lcinput = {}
+        #find an unused directory
+        project_folder = 'simulation_dir_'
+        idx = 0
+        while os.path.isdir(project_folder+str(idx)) is True:
+            idx += 1
+        self.pc.project_folder = project_folder+str(idx)
+
+
+    def add_lc(self, lc, name, errorbar_variance = True, errorbar_rescale = True):
+        '''
+        add a single light curve to the simulation
+        :param lc: numpy light curve array (N x 3) where each of the three columns are the time, flux, errorbars
+        :param name: reference name of light curve
+        :param errorbar_variance: TRUE/FALSE optimise error bars by a constant additive variance parameter
+        :param errorbar_rescale: TRUE/FALSE optimise error bars by a multiplicative rescaling parameter
+        :return:
+        '''
+
+        #record the input lightcurve
+        self.lcinput[name] = lc
+
+        #configure the error bar adjustment parameters
+        errorbar_parameters = []
+        if errorbar_variance is True:
+            errorbar_parameters.append('var')
+        if errorbar_rescale is True:
+            errorbar_parameters.append('multiplicative')
+
+        #add the light curve to the pycecream instance
+        if self.numlc == 0:
+            sharelag = False
+        else:
+            sharelag = True
+        self.pc.add_lc(lc, name=name, wavelength=-1.0,
+                       expand_errors = errorbar_parameters,
+                       share_previous_lag=sharelag, tophat_centroid_step=0.0)
+        self.numlc += 1
+
+
+
+    def run(self,ncores = 1):
+        '''
+        run the dream-pycecream simulation
+        :return:
+        '''
+        #setup and run the simplified pycecream simulation
+        self.pc.p_inclination_step=0.0
+        self.pc.N_iterations = self.N_iterations
+        self.pc.run(ncores=ncores)
+        #self._postrun()
+        self.light_curve_fits = self.pc.get_light_curve_fits()
+        self.MCMC_chains = self.pc.get_MCMC_chains()
+        self._newmerge()
+        os.system('rm -rf '+self.pc.project_folder)
+
+    def _postrun(self):
+        #return the individual output light curves and merge these into a single array
+        op = self.light_curve_fits(location=None)
+        self.lc_merged_individual = op['merged data']
+        self._op = self.__combine_individual_output_lightcurves()
+        self.lc_combined = self._op['combined_output']
+
+    def _newmerge(self):
+        '''
+
+        :return:
+        '''
+        # different way of merging
+        # needs to be integrated in to dream class
+        x = self.light_curve_fits
+        model = x['model']
+        merged_data = x['merged data']
+        cols = list(merged_data.keys())
+        reference_lc_name = cols[0]#'1m004lsc'
+        reference_time = model.values[:, 0]
+        reference_lc = model[reference_lc_name + ' model'].values[:]
+
+        for m in cols:
+            dat = merged_data[m]
+            t = dat[:, 0]
+            dat[:, 1] = np.interp(t, reference_time, reference_lc)
+            merged_data[m] = dat
+        self.lc_merged_individual = merged_data
+        self._op = self.__combine_individual_output_lightcurves()
+        self.lc_combined = self._op['combined_output']
+
+
+    def __combine_individual_output_lightcurves(self):
+        '''
+        combine the individual merged light curves (now on the correct scsale)
+        into a single light curve
+        :return:
+        '''
+        datinput = np.zeros((0, 3))
+        datmerged = np.zeros((0, 3))
+        for name, lc in self.lc_merged_individual.items():
+            datmerged = np.vstack([datmerged,self.lc_merged_individual[name]])
+            datinput = np.vstack([datinput, self.lcinput[name]])
+        return {'combined_input':datinput[np.argsort(datinput[:, 0]), :],
+            'combined_output':datmerged[np.argsort(datmerged[:, 0]), :]}
+
+
+    def _plot_individual(self, fig_in = None, ax_in = None, lightcurves = 'input'):
+        '''
+        plot the input light curves (one colour per light curve
+        :return:
+        '''
+        assert(lightcurves in ['input','merged'])
+        if lightcurves == 'input':
+            data = self.lcinput
+        else:
+            data = self.lc_merged_individual
+        names = list(data.keys())
+        if fig_in is None:
+            fig = plt.figure()
+        else:
+            fig = fig_in
+        if ax_in is None:
+            ax1 = fig.add_subplot(111)
+        else:
+            ax1 = ax_in
+        ax1.set_xlabel('Time')
+        ax1.set_ylabel('Flux')
+        for n in names:
+            dat = data[n]
+            color = next(ax1._get_lines.prop_cycler)['color']
+
+            #if plotting the merged light curves want to include initial error bars for comparison
+            if lightcurves == 'merged':
+                Nt = len(dat)
+                lc_in = self.lcinput[n]
+                lc_out = self.lc_merged_individual[n]
+                for it in range(Nt):
+                    if it == 0:
+                        label = n
+                    else:
+                        label = None
+                    if lc_in[it, 2] >= lc_out[it, 2]:
+                        order_in = 1
+                        order_out = 2
+                    else:
+                        order_in = 2
+                        order_out = 1
+                    ax1.errorbar([lc_out[it, 0]], [lc_out[it, 1]], [lc_in[it, 2]],
+                                 ls='', marker='', color=color, zorder=order_in, label=label)
+                    ax1.errorbar([lc_out[it, 0]], [lc_out[it, 1]], [lc_out[it, 2]],
+                                 ls='', marker='', color=color, alpha = 0.4,
+                                 zorder=order_out, label=None)
+
+            #if plotting the input light curves just plot the light curves
+            else:
+                ax1.errorbar(dat[:,0], dat[:,1], dat[:,2], ls = '', marker = None, color=color, label = n)
+            #if lightcurves == 'merged':
+            #    ax1.errorbar(dat[:, 0], dat[:, 1], dat[:, 2], ls='',
+            #                 marker=None, color=color, alpha = 0.4,label=None)
+
+        return [fig, ax1]
+
+
+    def plot_merged_combined(self, fig_in = None, ax_in = None):
+        '''
+        plot the merged light curves return figure, axis object
+        :return:
+        '''
+        lc_out = self._op['combined_output']
+        lc_in = self._op['combined_input']
+        if fig_in is None:
+            fig = plt.figure()
+        else:
+            fig = fig_in
+        if ax_in is None:
+            ax1 = fig.add_subplot(111)
+        else:
+            ax1 = ax_in
+        ax1.set_xlabel('Time')
+        ax1.set_ylabel('Flux')
+        Nt = len(lc_in)
+
+        #identify the correct plotting order (smaller error bars should be plotted on to)
+        for it in range(Nt):
+            if lc_in[it,2] >= lc_out[it,2]:
+                order_in = 1
+                order_out = 2
+            else:
+                order_in = 2
+                order_out = 1
+            ax1.errorbar([lc_out[it,0]], [lc_out[it,1]], [lc_in[it,2]],
+                     ls='',marker='',color='k',zorder=order_in, label=None)
+            ax1.errorbar([lc_out[it, 0]], [lc_out[it, 1]], [lc_out[it, 2]],
+                     ls='', marker='', color='grey', zorder=order_out, label=None)
+        return [fig, ax1]
+
+
+    def plot_merged_individual(self, fig_in = None, ax_in = None):
+        '''
+        plot the merged light curves return figure, axis object
+        :return:
+        '''
+        return self._plot_individual(fig_in = fig_in, ax_in = ax_in, lightcurves='merged')
+
+    def plot_input_individual(self, fig_in = None, ax_in = None):
+        '''
+        plot the merged light curves return figure, axis object
+        :return:
+        '''
+        return self._plot_individual(fig_in = fig_in, ax_in = ax_in, lightcurves='input')
+
+    def plot_chains_or_covariances(self,output_chains, Parm, ParmNicename,
+                                   type = 'covarainces', fig_in = None):
+        '''
+
+        :return:
+        '''
+        assert(type in ['covariances','chain'])
+
+        # make covariance corner plots
+        colnames_output_chains = list(output_chains.columns)
+        corner_columns = [c for c in colnames_output_chains if Parm in c]
+        new_corner_columns = [c.replace(Parm, '') for c in corner_columns]
+        df = output_chains[corner_columns].copy()
+        df.columns = new_corner_columns
+
+        # Isolate parameters optimised in MCMC chain
+        # if non of the current parameter set are varied then skip
+        dfstd = df.std()
+        names_VariedColumns = list(dfstd[dfstd > 1.e-10].index)
+        if len(names_VariedColumns) > 0:
+            # make the corner covariance plot and add title
+            print('making corner plots for ', names_VariedColumns)
+            if type == 'covariances':
+                #fig = corner.corner(df[names_VariedColumns], plot_contours=False,
+                #                    quantiles=[0.16, 0.5, 0.84],
+                #                    show_titles=True, title_kwargs={"fontsize": 12},
+                #                    truths=None)
+                g = sns.pairplot(df[names_VariedColumns], corner=True)
+                g.map_lower(corrfunc)
+                fig = g.fig
+                fig.suptitle('Covariance: ' + ParmNicename, fontsize=16)
+                fig.tight_layout()
+            elif type == 'chain':
+                if fig_in is not None:
+                    fig = fig_in
+                else:
+                    fig = plt.figure()
+                n = len(names_VariedColumns)
+                ncols = 2
+                nrows = int(np.ceil(n/ncols))
+                for i in range(n):
+                    ax1 = fig.add_subplot(nrows,ncols,i+1)
+                    ax1.plot(df[names_VariedColumns[i]].values)
+                    ax1.set_title(names_VariedColumns[i])
+                fig.suptitle('MCMC Chains: ' + ParmNicename, fontsize=16)
+
+            return fig
+
+
+
+def corrfunc(x,y, ax=None, **kws):
+    """Plot the correlation coefficient in the top left hand corner of a plot."""
+    #r, _ = pearsonr(x, y)
+    r = np.corrcoef(x,y)[0, 1]
+    ax = ax or plt.gca()
+    # Unicode for lowercase rho (ρ)
+    rho = '\u03C1'
+    ax.annotate(f'{rho} = {r:.2f}', xy=(.1, .9), xycoords=ax.transAxes)
+
+
+
+
+def _run_cmd(cmd):
+    '''
+    run the fortran code
+    :param cmd:
+    :return:
+    '''
+    os.system(cmd)
 
 
 
